/* This is a GTK+ module which makes menus translucent.
 *
 * Usage: export GTK_MODULES=<fullpath>/libgtkamber.so
 *
 * Author: nxin@netease.com
 */

//  Modified by Smart Lu for SRecite
 

#include <gdk-pixbuf/gdk-pixbuf.h>
#include "srecite.h"

#define BACK_ALPHA	30	/* 0 - 255 */

static gchar *back_pixbuf_key = "gtkamber-back-pixbuf";

/* Get background underneath the window */
static GdkPixbuf *
get_background(GdkWindow *window)
{
  int        x, y, width, height;
  GdkWindow *root;
  GdkPixbuf *pixbuf;
  guchar    *pixels;
  int        rowstride;
  int        x1, y1;

  /* Get snapshot from screen */
  gdk_window_get_origin (window, &x, &y);
  gdk_window_get_geometry (window, NULL, NULL, &width, &height, NULL);
  
  root = gdk_get_default_root_window ();
  pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, width, height);
  gdk_pixbuf_get_from_drawable (pixbuf, root, NULL,
				x, y, 0, 0, width, height);

  /* Set alpha channel */
  pixels = gdk_pixbuf_get_pixels (pixbuf);
  rowstride = gdk_pixbuf_get_rowstride (pixbuf);

  for (y1 = 0; y1 < height; y1++) {
    guchar *p;

    p = pixels + y1 * rowstride;
    for (x1 = 0; x1 < width; x1++) {
      p[3] = BACK_ALPHA;
      p += 4;
    }
  }

  return pixbuf;
}


static gboolean
menu_map_hook (GSignalInvocationHint *ihint,
	       guint                  n_param_values,
	       const GValue          *param_values,
	       gpointer               data)
{
  GtkWidget *menu;
  int x, y, width, height;
  GdkPixbuf *back_pixbuf;

  menu = g_value_peek_pointer (&param_values[0]);

  if (GTK_IS_LABEL(menu) || (GTK_IS_MENU (menu) && !gtk_menu_get_tearoff_state (GTK_MENU (menu)) ) ){
    if (!g_object_get_data (G_OBJECT (menu), back_pixbuf_key)) {
      gdk_window_get_origin (menu->window, &x, &y);
      gdk_window_get_geometry (menu->window, NULL, NULL, &width, &height, NULL);

//g_print ("menu map: %p, %d, %d, %d, %d\n", menu, x, y, width, height);

      back_pixbuf = get_background (menu->window);
      g_object_set_data_full (G_OBJECT(menu), back_pixbuf_key,
        back_pixbuf, (GDestroyNotify)g_object_unref);
    }
  }
  return TRUE;
}


static gboolean
menu_unmap_hook (GSignalInvocationHint *ihint,
		 guint                  n_param_values,
		 const GValue          *param_values,
		 gpointer               data)
{
  GtkWidget *widget;

  widget = g_value_peek_pointer (&param_values[0]);

  if (GTK_IS_LABEL(widget) || (GTK_IS_MENU (widget)) ){
//    g_print ("unmap, %p, type: %s\n", widget, g_type_name (GTK_WIDGET_TYPE (widget)));
    g_object_set_data (G_OBJECT (widget), back_pixbuf_key, NULL);
  }

  return TRUE;
}

static gboolean
menu_item_expose_hook (GSignalInvocationHint *ihint,
		       guint                  n_param_values,
		       const GValue          *param_values,
		       gpointer               data)
{
  GtkWidget      *widget;
  GdkEventExpose *event;
  GtkWidget *menu;

  widget = g_value_peek_pointer (&param_values[0]);
  event = g_value_peek_pointer (&param_values[1]);
  menu = gtk_widget_get_parent (widget);

  if (GTK_IS_LABEL(menu) || (GTK_IS_MENU(menu) && !gtk_menu_get_tearoff_state (GTK_MENU (menu))) ){
    GdkPixbuf *back_pixbuf;
    int x_menu, y_menu, x_item, y_item;

    back_pixbuf = g_object_get_data (G_OBJECT (menu), back_pixbuf_key);

    if (back_pixbuf) {

      gdk_window_get_origin (menu->window, &x_menu, &y_menu);
      gdk_window_get_origin (widget->window, &x_item, &y_item);

      gdk_pixbuf_render_to_drawable (back_pixbuf, widget->window,
        NULL,
        event->area.x + x_item - x_menu, event->area.y + y_item - y_menu,
        event->area.x, event->area.y,
        event->area.width, event->area.height,
        GDK_RGB_DITHER_NONE, 0, 0);
    }
  }

  return TRUE;
}

/* init transparent WINDOW and MENU */
void transparent_init ()
{
  /* Check version */
  if (gtk_major_version != GTK_MAJOR_VERSION)
    return;

  /* Make class GtkMenu and GtkMenuItem available */
  g_type_class_ref (GTK_TYPE_MENU);
  g_type_class_ref (GTK_TYPE_MENU_ITEM);
  g_type_class_ref (GTK_TYPE_LABEL);
  //g_type_class_ref (GTK_TYPE_WINDOW);

  /* Add hooks */
  g_signal_add_emission_hook (g_signal_lookup ("map", GTK_TYPE_MENU),
    0, menu_map_hook, NULL, NULL);

  g_signal_add_emission_hook (g_signal_lookup ("unmap", GTK_TYPE_MENU),
    0, menu_unmap_hook, NULL, NULL);

  g_signal_add_emission_hook (g_signal_lookup ("expose_event", GTK_TYPE_MENU_ITEM),
    0, menu_item_expose_hook, NULL, NULL);
/*  
  g_signal_add_emission_hook (g_signal_lookup ("map", GTK_TYPE_LABEL),
    0, menu_map_hook, NULL, NULL);

  g_signal_add_emission_hook (g_signal_lookup ("unmap", GTK_TYPE_LABEL),
    0, menu_unmap_hook, NULL, NULL);

  g_signal_add_emission_hook (g_signal_lookup ("expose_event", GTK_TYPE_LABEL),
    0, menu_item_expose_hook, NULL, NULL);
*/
}
